#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <fcntl.h>
#include <unistd.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <netdb.h>
#include <arpa/inet.h> 
#include <pthread.h>
#include "event.h"
#include "event_queue.h"

#define UDP_SOCKET_IP "127.0.0.1"

EventQueue *eq;

int count_pipe_events = 0;
int pipe2_id = 0;

int32_t key_handler(void *p, void *d)
{
  printf("Key: %s\n",(char*)d);
  return 0;
}

int32_t socket_handler(void *p, void *d)
{
  printf("Socket: %s\n",(char*)d);
  return 0;
}

int32_t socket_handler_2(void *p, void *d)
{
  printf("Socket2: %s\n",(char*)d);
  return 0;
}

int32_t pipe_handler(void *p, void *d)
{
  count_pipe_events++;
  if ( count_pipe_events == 6 ) remove_event(eq, pipe2_id);

  printf("Pipe: %s\n",(char*)d);
  return 0;
}

int32_t pipe_handler_2(void *p, void *d)
{
  printf("Pipe2: %s\n",(char*)d);
  return 0;
}

int32_t time_handler(void *p, void *d)
{
  insert_event(eq, SOCKET, 0, socket_handler_2, NULL);
  printf("Time: time = %d\n",get_time(eq));
  return 0;
}

int32_t timeout_handler(void *p, void *d)
{
  printf("Stop event queue\n");
  stop_event_queue(eq);
  return 0;
}


void test_thread(void *argv)
{

  /* PIPE */
  int np_fd = open(NAMED_PIPE_PATH, O_WRONLY | O_SYNC);
  if (np_fd == -1) {
    printf("Couldn't open pipe");
    exit(0);
  }

  /* SOCKET */
  int sd;
  struct sockaddr_in server;
  struct  hostent *hp, *gethostbyname();

  sd = socket (AF_INET,SOCK_DGRAM,0);

  server.sin_family = AF_INET;
  hp = gethostbyname(UDP_SOCKET_IP);
  bcopy ( hp->h_addr, (struct sockaddr *)&(server.sin_addr.s_addr), hp->h_length);
  server.sin_port = htons(UDP_SOCKET_PORT);

  /* TEST */

  char buf[512];
  int i;

  usleep(1300*1000);

  /* Pipe Test */
  printf("-------- Test 1 --------\n");

  write(np_fd, "pipe event 1\n", strlen("pipe event 1\n")+1);

  usleep(500*1000);

  /* Socket Test */ 
  printf("-------- Test 2 --------\n");

  sendto(sd, "socket event 1\n", strlen("socket event 1\n"), 0, (struct sockaddr *)&server, sizeof(server));

  usleep(500*1000);

  printf("-------- Test 3 --------\n");
  for( i = 0; i < 5; i++ ) {
    sprintf(buf,"pipe event %d\n",(i+2));
    write(np_fd, buf, strlen(buf)+1);
    usleep(500*1000);
  }

  /* 4.5 */
  printf("-------- Test 4 --------\n");
  sprintf(buf,"time = %d\n",get_time(eq));

  write(np_fd, buf, strlen(buf)+1);
  sendto(sd, buf, strlen(buf), 0, (struct sockaddr *)&server, sizeof(server));

  usleep(1000*1000);

  /* 5.5 */
  printf("-------- Test 5 --------\n");
  sprintf(buf,"time = %d\n",get_time(eq));

  write(np_fd, buf, strlen(buf)+1);
  sendto(sd, buf, strlen(buf), 0, (struct sockaddr *)&server, sizeof(server));

  usleep(1000*1000);

  printf("-------- End --------\n");

  while ( eq->state == RUNNING ) usleep(100*1000);

  close(sd);
  close(np_fd); 
}

int main(int argc, char **argv)
{
  eq = new_event_queue();

  insert_time_event(eq, 0, timeout_handler, NULL, 10);
  insert_time_event(eq, 0, time_handler, NULL, 5);

  insert_event(eq, KEY, 0, key_handler, NULL);
  insert_event(eq, SOCKET, 0, socket_handler, NULL);

  insert_event(eq, PIPE, 0, pipe_handler, NULL);
  pipe2_id = insert_event(eq, PIPE, 0, pipe_handler_2, NULL);

  void *thread_1_join;
  pthread_t thread_1;
  pthread_create(&thread_1,(pthread_attr_t*)NULL,(void *)&test_thread,(void*)NULL);

  run_event_queue(eq);

  delete_event_queue(eq);

  pthread_join(thread_1,&thread_1_join);

  return 0;
}
